@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2004 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -57,6 +58,9 @@ enum
   PROP_0,
   PROP_NAME,
   PROP_STATE,
+  PROP_STATS,
+  PROP_WIRELESS,
+  PROP_SIGNAL_STRENGTH,
   PROP_ERROR
 };
 
@@ -65,16 +69,15 @@ struct _NetstatusIfacePrivate
   char           *name;
 
   NetstatusState  state;
-  long            in_packets;
-  long            out_packets;
-  long            in_bytes;
-  long            out_bytes;
+  NetstatusStats  stats;
+  int             signal_strength;
   GError         *error;
 
   int             sockfd;
   guint           monitor_id;
 
   guint           error_polling : 1;
+  guint           is_wireless : 1;
 };
 
 static void     netstatus_iface_instance_init   (NetstatusIface      *iface,
@@ -161,6 +164,32 @@ netstatus_iface_class_init (NetstatusIfaceClass *klass)
 						      NETSTATUS_STATE_DISCONNECTED,
 						      G_PARAM_READABLE));
 
+  g_object_class_install_property (gobject_class,
+				   PROP_STATS,
+				   g_param_spec_boxed ("stats",
+						       _("Stats"),
+						       _("The interface packets/bytes statistics"),
+						       NETSTATUS_TYPE_STATS,
+						       G_PARAM_READABLE));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_WIRELESS,
+				   g_param_spec_boolean ("wireless",
+							 _("Wireless"),
+							 _("Whether the interface is a wireless interface"),
+							 FALSE,
+							 G_PARAM_READABLE));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_SIGNAL_STRENGTH,
+				   g_param_spec_int ("signal-strength",
+						     _("Signal"),
+						     _("Wireless signal strength percentage"),
+						     0,
+						     100,
+						     0,
+						     G_PARAM_READABLE));
+
   g_object_class_install_property (gobject_class,
 				   PROP_ERROR,
 				   g_param_spec_boxed ("error",
@@ -234,6 +263,15 @@ netstatus_iface_get_property (GObject    *object,
     case PROP_STATE:
       g_value_set_enum (value, iface->priv->state);
       break;
+    case PROP_STATS:
+      g_value_set_boxed (value, &iface->priv->stats);
+      break;
+    case PROP_WIRELESS:
+      g_value_set_boolean (value, iface->priv->is_wireless);
+      break;
+    case PROP_SIGNAL_STRENGTH:
+      g_value_set_int (value, iface->priv->signal_strength);
+      break;
     case PROP_ERROR:
       g_value_set_boxed (value, iface->priv->error);
     default:
@@ -292,29 +330,29 @@ netstatus_iface_get_state (NetstatusIface *iface)
 }
 
 void
-netstatus_iface_get_packet_statistics (NetstatusIface *iface,
-				       long           *in,
-				       long           *out)
+netstatus_iface_get_statistics (NetstatusIface *iface,
+				NetstatusStats *stats)
 {
   g_return_if_fail (NETSTATUS_IS_IFACE (iface));
 
-  if (in)
-    *in  = iface->priv->in_packets;
-  if (out)
-    *out = iface->priv->out_packets;
+  if (stats)
+    *stats  = iface->priv->stats;
 }
 
-void
-netstatus_iface_get_byte_statistics (NetstatusIface *iface,
-				     long           *in,
-				     long           *out)
+gboolean
+netstatus_iface_get_is_wireless (NetstatusIface *iface)
 {
-  g_return_if_fail (NETSTATUS_IS_IFACE (iface));
+  g_return_val_if_fail (NETSTATUS_IS_IFACE (iface), FALSE);
 
-  if (in)
-    *in = iface->priv->in_bytes;
-  if (out)
-    *out = iface->priv->out_bytes;
+  return iface->priv->is_wireless;
+}
+
+int
+netstatus_iface_get_signal_strength (NetstatusIface *iface)
+{
+  g_return_val_if_fail (NETSTATUS_IS_IFACE (iface), 0);
+
+  return iface->priv->signal_strength;
 }
 
 void
@@ -484,8 +522,8 @@ netstatus_iface_poll_state (NetstatusIface *iface)
 	   in_bytes, out_bytes,
 	   iface->priv->in_bytes, iface->priv->out_bytes);
   
-  rx = in_packets  > iface->priv->in_packets;
-  tx = out_packets > iface->priv->out_packets;
+  rx = in_packets  > iface->priv->stats.in_packets;
+  tx = out_packets > iface->priv->stats.out_packets;
 
   if (!tx && !rx)
     state = NETSTATUS_STATE_IDLE;
@@ -496,16 +534,47 @@ netstatus_iface_poll_state (NetstatusIface *iface)
   else /* if (rx) */
     state = NETSTATUS_STATE_RX;
 
-  iface->priv->in_packets  = in_packets;
-  iface->priv->out_packets = out_packets;
-  iface->priv->in_bytes    = in_bytes;
-  iface->priv->out_bytes   = out_bytes;
-
   dprintf (POLLING, "State: %s\n", netstatus_get_state_string (state));
 
+  if (tx || rx)
+    {
+      iface->priv->stats.in_packets  = in_packets;
+      iface->priv->stats.out_packets = out_packets;
+      iface->priv->stats.in_bytes    = in_bytes;
+      iface->priv->stats.out_bytes   = out_bytes;
+
+      g_object_notify (G_OBJECT (iface), "stats");
+    }
+
   return state;
 }
 
+static gboolean
+netstatus_iface_poll_wireless_details (NetstatusIface *iface,
+				       int            *signal_strength)
+{
+  char     *error_message;
+  gboolean  is_wireless;
+
+  error_message = netstatus_sysdeps_read_iface_wireless_details (iface->priv->name,
+								 &is_wireless,
+								 signal_strength);
+
+  if (error_message)
+    {
+      netstatus_iface_set_polling_error (iface,
+					 NETSTATUS_ERROR_WIRELESS_DETAILS,
+					 error_message);
+      g_free (error_message);
+      
+      return FALSE;
+    }
+
+  netstatus_iface_clear_error (iface, NETSTATUS_ERROR_WIRELESS_DETAILS);
+
+  return is_wireless;
+}
+
 static void
 netstatus_iface_increase_poll_delay_in_error (NetstatusIface *iface)
 {
@@ -544,6 +613,8 @@ static gboolean
 netstatus_iface_monitor_timeout (NetstatusIface *iface)
 {
   NetstatusState state;
+  int            signal_strength;
+  gboolean       is_wireless;
  
   state = netstatus_iface_poll_state (iface);
 
@@ -554,6 +625,19 @@ netstatus_iface_monitor_timeout (NetstatusIface *iface)
       g_object_notify (G_OBJECT (iface), "state");
     }
 
+  is_wireless = netstatus_iface_poll_wireless_details (iface, &signal_strength);
+  if (iface->priv->is_wireless != is_wireless)
+    {
+      iface->priv->is_wireless = is_wireless;
+      g_object_notify (G_OBJECT (iface), "wireless");
+    }
+
+  if (iface->priv->signal_strength != signal_strength)
+    {
+      iface->priv->signal_strength = signal_strength;
+      g_object_notify (G_OBJECT (iface), "signal-strength");
+    }
+
   netstatus_iface_increase_poll_delay_in_error (iface);
   
   return TRUE;
@@ -562,10 +646,18 @@ netstatus_iface_monitor_timeout (NetstatusIface *iface)
 static void
 netstatus_iface_init_monitor (NetstatusIface *iface)
 {
-  iface->priv->in_packets  = 0;
-  iface->priv->out_packets = 0;
-  iface->priv->in_bytes    = 0;
-  iface->priv->out_bytes   = 0;
+  iface->priv->stats.in_packets  = 0;
+  iface->priv->stats.out_packets = 0;
+  iface->priv->stats.in_bytes    = 0;
+  iface->priv->stats.out_bytes   = 0;
+  iface->priv->signal_strength   = 0;
+  iface->priv->is_wireless       = FALSE;
+
+  g_object_freeze_notify (G_OBJECT (iface));
+  g_object_notify (G_OBJECT (iface), "state");
+  g_object_notify (G_OBJECT (iface), "wireless");
+  g_object_notify (G_OBJECT (iface), "signal-strength");
+  g_object_thaw_notify (G_OBJECT (iface));
 
   if (iface->priv->monitor_id)
     {
