@@ -30,6 +30,7 @@ G_BEGIN_DECLS
 
 #define NETSTATUS_ERROR        (netstatus_error_quark ())
 #define NETSTATUS_TYPE_G_ERROR (netstatus_g_error_get_type ())
+#define NETSTATUS_TYPE_STATS   (netstatus_stats_get_type ())
 
 typedef enum {
   NETSTATUS_STATE_DISCONNECTED = 0,
@@ -59,8 +60,17 @@ typedef enum {
   NETSTATUS_DEBUG_POLLING = 1 << 0
 } NetstatusDebugFlags;
 
+typedef struct
+{
+  long in_packets;
+  long out_packets;
+  long in_bytes;
+  long out_bytes;
+} NetstatusStats;
+
 GQuark               netstatus_error_quark                (void);
 GType                netstatus_g_error_get_type           (void);
+GType                netstatus_stats_get_type             (void);
 void                 netstatus_adopt_error                (GError         *error,
 							   NetstatusError  code);
 
